@@ -7,19 +7,18 @@ use_rabbitmq = config.settings['Rabbitmq']['active']
 if use_rabbitmq:
     import pika
 
-use_message_pipe = config.settings['Message Pipe']['active']
-if use_message_pipe:
-    import telnetlib
-
 use_http_post = config.settings['Http Post']['active']
 if use_http_post:
     import urllib
 
+def format_http_url(settings):
+    host = settings['Http Post']['server']
+    port = settings['Http Post']['port']
+    return "http://%s:%s/publish" % (host, port)
+
+http_url = format_http_url(config.settings)
 
 def update_http(anm = None, nidb = None):
-    host = config.settings['Http Post']['server']
-    port = config.settings['Http Post']['port']
-    http_url = "http://%s:%s/publish" % (host, port)
 
     if anm and nidb:
         body = autonetkit.ank_json.dumps(anm, nidb)
@@ -30,7 +29,8 @@ def update_http(anm = None, nidb = None):
         body = json.dumps({}) # blank to test visualisation server running
 
     params = urllib.urlencode({
-        'body': body
+        'body': body,
+        'type': 'anm',
         })
     try:
         data = urllib.urlopen(http_url, params).read()
@@ -43,7 +43,10 @@ def update_http(anm = None, nidb = None):
         log.info("Visualisation server running")
 
 
-def highlight(nodes, edges):
+def highlight(nodes, edges, paths = None):
+    if not paths:
+        paths = []
+
     def nfilter(n):
         try:
             return n.id
@@ -58,28 +61,50 @@ def highlight(nodes, edges):
 
     nodes = [nfilter(n) for n in nodes]
     edges = [efilter(e) for e in edges]
+    filtered_paths = []
+    for path in paths:
+        #TODO: tidy this logic
+        if isinstance(path, dict) and 'path' in path:
+            path_data = path # use as-s
+        else:
+            import random
+            is_verified = bool(random.randint(0,1))
+            path_data = {'path': path,
+                    'verified': is_verified}
+
+        path_data['path'] = [nfilter(n) for n in path_data['path']]
+        filtered_paths.append(path_data)
+
+    #TODO: remove "highlight" from json, use as url params to distinguish
     import json
     body = json.dumps({
-        'highlight': {
         'nodes': nodes,
         'edges': edges,
-        }
+        'paths': filtered_paths,
         })
 
     params = urllib.urlencode({
-        'body': body
+        'body': body,
+        'type': 'highlight',
         })
 
     #TODO: split this common function out, create at runtime so don't need to keep reading config
-    host = config.settings['Http Post']['server']
-    port = config.settings['Http Post']['port']
-    http_url = "http://%s:%s/publish" % (host, port)
     try:
         data = urllib.urlopen(http_url, params).read()
     except IOError, e:
         log.info("Unable to connect to HTTP Server %s: e" % (http_url, e))
 
+def publish_data(data, type_key):
+    params = urllib.urlencode({
+        'body': data,
+        'type': type_key,
+        })
 
+    #TODO: split this common function out, create at runtime so don't need to keep reading config
+    try:
+        data = urllib.urlopen(http_url, params).read()
+    except IOError, e:
+        log.info("Unable to connect to HTTP Server %s: %s" % (http_url, e))
 
 class AnkMessaging(object):
 
@@ -95,14 +120,6 @@ class AnkMessaging(object):
                 self.publish = self.publish_pika
                 self.publish_compressed = self.publish_compressed_pika
 
-            if use_message_pipe:
-                #TODO: make message server also settable
-                port = config.settings['Message Pipe']['port']
-                self.telnet_port = port
-                self.publish = self.publish_telnet
-                self.publish_compressed = self.publish_telnet
-#TODO: support use of both at once....
-
             if use_http_post:
                 host = config.settings['Http Post']['server']
                 port = config.settings['Http Post']['port']
@@ -110,7 +127,7 @@ class AnkMessaging(object):
                 self.publish = self.publish_http_post
                 self.publish_compressed = self.publish_http_post
 
-            if not (use_rabbitmq or use_message_pipe or use_http_post):
+            if not (use_rabbitmq or use_http_post):
                 log.debug("Not using Rabbitmq or telnet")
                 self.publish = self.publish_blank_stub
                 self.publish_compressed = self.publish_blank_stub
@@ -142,14 +159,6 @@ class AnkMessaging(object):
         data = json.dumps(body, cls=autonetkit.ank_json.AnkEncoder, indent = 4)
         self.publish(None, None, data)
 
-    def publish_telnet(self, exchange, routing_key, body):
-        try:
-            tn = telnetlib.Telnet("localhost", self.telnet_port)
-            tn.write(body)
-            tn.close()
-        except socket.error:
-            log.warning("Unable to connect to telnet on localhost at %s" % self.telnet_port)
-
     def publish_compressed_telnet(self, exchange, routing_key, body):
         import zlib
 #TODO: note don't compress - no upper bound if telnet sockets
@@ -173,8 +182,8 @@ class AnkMessaging(object):
 #TODO: log that not sending for debug purposes
         return
 
-
     def publish_http_post(self, exchange, routing_key, body):
+        print "called"
         params = urllib.urlencode({
             'body': body
             })
@@ -185,5 +194,4 @@ class AnkMessaging(object):
 
         #print data # can log response
 
-
-
+#TODO: write new module that sends to webserver and takes parameter to distinguish eg starting, ip_allocations, etc
