@@ -11,6 +11,8 @@ import networkx as nx
 import autonetkit.ank as ank_utils
 import itertools
 
+#TODO: remove retain edge_id once removed from compiler
+
 __all__ = ['build']
 
 MESSAGING = ank_messaging.AnkMessaging()
@@ -47,6 +49,7 @@ def grid_2d(dim):
         graph.node[n]['platform'] = 'cisco'
         graph.node[n]['syntax'] = 'ios2'
         graph.node[n]['host'] = 'internal'
+        graph.node[n]['ibgp_level'] = 0
 
     mapping = {n: "%s_%s" % (n[0], n[1]) for n in graph}
     nx.relabel_nodes(graph, mapping, copy=False) # Networkx wipes data if remap with same labels
@@ -63,8 +66,7 @@ def grid_2d(dim):
 
     return graph
 
-def build(input_graph):
-    """Main function to build network overlay topologies"""
+def initialise(input_graph):
     anm = autonetkit.anm.AbstractNetworkModel()
 
     input_undirected = nx.Graph(input_graph)
@@ -89,18 +91,22 @@ def build(input_graph):
         g_in.nodes("is_router", platform="junosphere"), syntax="junos")
     g_in.update(g_in.nodes("is_router", platform="dynagen"), syntax="ios")
     g_in.update(g_in.nodes("is_router", platform="netkit"), syntax="quagga")
+    g_in.update(g_in.nodes("is_server", platform="netkit"), syntax="quagga")
 
     g_graphics = anm.add_overlay("graphics")  # plotting data
     g_graphics.add_nodes_from(g_in, retain=['x', 'y', 'device_type',
                               'device_subtype', 'pop', 'asn'])
 
+    return anm
+
+def apply_design_rules(anm):
+    g_in = anm['input']
+
     build_phy(anm)
-    autonetkit.update_http(anm)
     g_phy = anm['phy']
 
-    build_vrf(anm)
+    build_vrf(anm) # need to do before to add loopbacks before ip allocations
     build_ip(anm) # ip infrastructure topology
-    autonetkit.update_http(anm)
 
     address_family = g_in.data.address_family or "v4" # default is v4
 #TODO: can remove the infrastructure now create g_ip seperately
@@ -114,15 +120,14 @@ def build(input_graph):
     if address_family in ("v6", "dual_stack"):
         build_ipv6(anm)
         g_phy.update(g_phy, use_ipv6 = True)
+    else:
+        anm.add_overlay("ipv6") # placeholder for compiler logic
 
     default_igp = g_in.data.igp or "ospf" 
     non_igp_nodes = [n for n in g_in if not n.igp]
 #TODO: should this be modifying g_in?
     g_in.update(non_igp_nodes, igp=default_igp) # store igp onto each node
 
-    anm.add_overlay("ospf")
-    anm.add_overlay("isis")
-
     ank_utils.copy_attr_from(g_in, g_phy, "include_csr")
 
     build_ospf(anm)
@@ -130,8 +135,29 @@ def build(input_graph):
     build_bgp(anm)
     autonetkit.update_http(anm)
 
+# post-processing
+    mark_ebgp_vrf(anm)
+    build_ibgp_vpn_v4(anm) # build after bgp as is based on
+    #autonetkit.update_http(anm)
     return anm
 
+
+def build(input_graph):
+    """Main function to build network overlay topologies"""
+    anm = initialise(input_graph)
+    anm = apply_design_rules(anm)
+    return anm
+
+def vrf_pre_process(anm):
+    """Marks nodes in g_in as appropriate based on vrf roles.
+    CE nodes -> ibgp_level = 0, so not in ibgp (this is allocated later)
+    """
+    log.debug("Applying VRF pre-processing")
+    g_vrf = anm['vrf']
+    for node in g_vrf.nodes(vrf_role = "CE"):
+        log.debug("Marking CE node %s as non-ibgp" % node)
+        node['input'].ibgp_level = 0
+
 def allocate_vrf_roles(g_vrf):
     """Allocate VRF roles"""
     g_phy = g_vrf.anm['phy']
@@ -146,7 +172,7 @@ def allocate_vrf_roles(g_vrf):
     for node in non_ce_nodes:
         phy_neighbors = g_phy.node(node).neighbors("is_router")  
         # neighbors from physical graph for connectivity
-        phy_neighbors = [neigh for neigh in phy_neighbors if neigh.asn == node.asn]
+        phy_neighbors = [neigh for neigh in phy_neighbors]
             # filter to just this asn
         if any(g_vrf.node(neigh).vrf_role == "CE" for neigh in phy_neighbors):
             # phy neigh has vrf set in this graph
@@ -165,28 +191,134 @@ def add_vrf_loopbacks(g_vrf):
             node.add_loopback(vrf_name=vrf_name,
                               description="loopback for vrf %s" % vrf_name)
 
-    for node in g_vrf.nodes(vrf_role="CE"):
-        node.add_loopback(vrf_name = node.vrf_name,
-                          description="loopback for vrf %s" % node.vrf_name)
+def build_ibgp_vpn_v4(anm):
+    """Based on the ibgp_v4 hierarchy rules.
+    Exceptions:
+    1. Remove links to (PE, RRC) nodes
 
-def vrf_edges(g_vrf):
-    """Calculate edges for vrf overlay"""
-    g_phy = g_vrf.anm['phy']
-    for _, devices in g_phy.groupby("asn").items():
-        as_graph = g_phy.subgraph(devices)
-        edges = [(edge.src, edge.dst) for edge in as_graph.edges()]
-        edges_vrf = [(g_vrf.node(s), g_vrf.node(t)) for (s, t) in edges]
-        pe_to_ce_edges = []
-        ce_to_pe_edges = []
-        for src, dst in edges_vrf:
-            if (src.vrf_role, dst.vrf_role) == ("CE", "PE"):
-                pe_to_ce_edges.append((src, dst))
-                ce_to_pe_edges.append((dst, src))
-            if (src.vrf_role, dst.vrf_role) == ("PE", "CE"):
-                pe_to_ce_edges.append((dst, src))
-                ce_to_pe_edges.append((src, dst))
+    CE nodes are excluded from RR hierarchy ibgp creation through pre-process step
+
+    """
+    #TODO: remove the bgp layer and have just ibgp and ebgp
+    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
+    g_bgp = anm['bgp']
+    g_ibgp_v4 = anm['ibgp_v4']
+    g_vrf = anm['vrf']
+    g_phy = anm['phy']
+    g_ibgp_vpn_v4= anm.add_overlay("ibgp_vpn_v4", directed=True)
+
+    ibgp_v4_nodes = list(g_ibgp_v4.nodes())
+    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
+    pe_rrc_nodes = {n for n in ibgp_v4_nodes if
+            n in pe_nodes and n.ibgp_role == "RRC"}
+    ce_nodes = set(g_vrf.nodes(vrf_role = "CE"))
+
+    ibgp_vpn_v4_nodes = (n for n in ibgp_v4_nodes 
+            if n not in pe_rrc_nodes and n not in ce_nodes)
+    g_ibgp_vpn_v4.add_nodes_from(ibgp_vpn_v4_nodes, retain = "ibgp_level")
+    g_ibgp_vpn_v4.add_edges_from(g_ibgp_v4.edges(), retain = "direction")
+
+    for node in g_ibgp_vpn_v4:
+        if node.ibgp_level in (2, 3): # HRR or RR
+            node.retain_route_target = True
+
+    ce_edges = [e for e in g_ibgp_vpn_v4.edges()
+            if e.src in ce_nodes or e.dst in ce_nodes]
+
+    """
+    #TODO: do we still need this?
+    g_ibgp_vpn_v4.remove_edges_from(ce_edges)
+
+# add CE -> PE links based on physical connectivity
+#Note: this could later look at 
+    ce_to_pe_edges = []
+    ce_phy_nodes = {g_phy.node(n) for n in ce_nodes}
+    pe_phy_nodes = {g_phy.node(n) for n in g_vrf.nodes(vrf_role = "PE")}
+    ce_to_pe_edges += (e for e in g_phy.edges()
+            if 
+            e.src.asn == e.dst.asn 
+            and
+            ((e.src in ce_phy_nodes and e.dst in pe_phy_nodes)
+            or (e.src in pe_phy_nodes and e.dst in ce_phy_nodes)))
+
+    g_ibgp_vpn_v4.add_edges_from(ce_to_pe_edges, type="ibgp", bidirectional = True)
+    """
+
+    # mark ibgp direction
+    ce_pe_edges = []
+    pe_ce_edges = []
+    for edge in g_ibgp_vpn_v4.edges():
+        if (edge.src.vrf_role, edge.dst.vrf_role) == ("CE", "PE"):
+            edge.direction = "up"
+            edge.vrf = edge.src.vrf
+            ce_pe_edges.append(edge)
+        elif (edge.src.vrf_role, edge.dst.vrf_role) == ("PE", "CE"):
+            edge.direction = "down"
+            edge.vrf = edge.dst.vrf
+            pe_ce_edges.append(edge)
+
+
+    #TODO: Document this
+    g_ibgpv4 = anm['ibgp_v4']
+    g_ibgpv6 = anm['ibgp_v6']
+    g_ibgpv4.remove_edges_from(ce_edges)
+    g_ibgpv6.remove_edges_from(ce_edges)
+    g_ibgpv4.add_edges_from(ce_pe_edges, retain = ["direction", "vrf"])
+    g_ibgpv4.add_edges_from(pe_ce_edges, retain = ["direction", "vrf"])
+    g_ibgpv6.add_edges_from(ce_pe_edges, retain = ["direction", "vrf"])
+    g_ibgpv6.add_edges_from(pe_ce_edges, retain = ["direction", "vrf"])
+    for edge in pe_ce_edges:
+        # mark as exclude so don't include in standard ibgp config stanzas
+        if g_ibgpv4.has_edge(edge):
+            edge['ibgp_v4'].exclude = True
+        if g_ibgpv6.has_edge(edge):
+            edge['ibgp_v6'].exclude = True
+
+# legacy
+    g_bgp = anm['bgp']
+    g_bgp.remove_edges_from(ce_edges)
+    g_bgp.add_edges_from(ce_pe_edges, retain = ["direction", "vrf", "type"])
+    g_bgp.add_edges_from(pe_ce_edges, retain = ["direction", "vrf", "type"])
+
+
+    # also need to modify the ibgp_v4 and ibgp_v6 graphs
+
+def build_mpls_ldp(anm):
+    """Builds MPLS LDP"""
+    g_in = anm['input']
+    g_vrf = anm['vrf']
+    g_mpls_ldp = anm.add_overlay("mpls_ldp")
+    g_mpls_ldp.add_nodes_from(g_in.nodes("is_router"), retain=["vrf_role", "vrf"])
+
+    # store as set for faster lookup
+    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
+    p_nodes = set(g_vrf.nodes(vrf_role = "P"))
+
+    pe_to_pe_edges = (e for e in g_in.edges()
+            if e.src in pe_nodes and e.dst in pe_nodes)
+    g_mpls_ldp.add_edges_from(pe_to_pe_edges)
+    pe_to_p_edges = (e for e in g_in.edges()
+            if e.src in pe_nodes and e.dst in p_nodes
+            or e.src in p_nodes and e.dst in pe_nodes)
+    g_mpls_ldp.add_edges_from(pe_to_p_edges)
+
+def mark_ebgp_vrf(anm):
+    g_ebgp = anm['ebgp']
+    g_vrf = anm['vrf']
+    g_ebgpv4= anm['ebgp_v4']
+    g_ebgpv6= anm['ebgp_v6']
+    pe_nodes = set(g_vrf.nodes(vrf_role = "PE"))
+    ce_nodes = set(g_vrf.nodes(vrf_role = "CE"))
+    for edge in g_ebgpv4.edges():
+        if (edge.src in pe_nodes and edge.dst in ce_nodes):
+            edge.exclude = True # exclude from "regular" ebgp (as put into vrf stanza)
+            edge.vrf = edge.dst['vrf'].vrf
+
+    for edge in g_ebgpv6.edges():
+        if (edge.src in pe_nodes and edge.dst in ce_nodes):
+            edge.exclude = True # exclude from "regular" ebgp (as put into vrf stanza)
+            edge.vrf = edge.dst['vrf'].vrf
 
-    return pe_to_ce_edges, ce_to_pe_edges
 
 def build_vrf(anm):
     """Build VRF Overlay"""
@@ -196,16 +328,29 @@ def build_vrf(anm):
 
     allocate_vrf_roles(g_vrf)
 
+    vrf_pre_process(anm)
+
     def is_pe_ce_edge(edge):
         src_vrf_role = g_vrf.node(edge.src).vrf_role
         dst_vrf_role = g_vrf.node(edge.dst).vrf_role
         return (src_vrf_role, dst_vrf_role) in (("PE", "CE"), ("CE", "PE"))
 
     vrf_add_edges = (e for e in g_in.edges()
-            if e.src.asn == e.dst.asn and is_pe_ce_edge(e))
-
+           if is_pe_ce_edge(e))
+    #TODO: should mark as being towards PE or CE
     g_vrf.add_edges_from(vrf_add_edges, retain=['edge_id'])
 
+    def is_pe_p_edge(edge):
+        src_vrf_role = g_vrf.node(edge.src).vrf_role
+        dst_vrf_role = g_vrf.node(edge.dst).vrf_role
+        return (src_vrf_role, dst_vrf_role) in (("PE", "P"), ("P", "PE"))
+    vrf_add_edges = (e for e in g_in.edges()
+            if is_pe_p_edge(e))
+    g_vrf.add_edges_from(vrf_add_edges, retain=['edge_id'])
+
+    build_mpls_ldp(anm)
+    # add PE to P edges
+
     add_vrf_loopbacks(g_vrf)
     # allocate route-targets per AS
     # This could later look at connected components for each ASN
@@ -228,6 +373,10 @@ def build_vrf(anm):
         # Set the vrf of the edge to be that of the CE device (either src or dst)
         edge.vrf = edge.src.vrf if edge.src.vrf_role is "CE" else edge.dst.vrf
 
+    # map attributes to interfaces
+    for edge in g_vrf.edges():
+        for interface in edge.interfaces():
+            interface.vrf_name = edge.vrf
 
 def three_tier_ibgp_corner_cases(rtrs):
     """Calculate edges for iBGP l3 clusters that don't contain a HRR.
@@ -299,7 +448,6 @@ def three_tier_ibgp_edges(routers):
   
     return up_links, down_links, over_links
 
-
 def build_two_tier_ibgp(routers):
     """Constructs two-tier ibgp"""
     up_links = down_links = over_links = []
@@ -314,11 +462,83 @@ def build_two_tier_ibgp(routers):
                   and s.ibgp_l3_cluster == t.ibgp_l3_cluster]
     return up_links, down_links, over_links
 
+def build_ibgp_v4(anm):
+    #TODO: remove the bgp layer and have just ibgp and ebgp
+    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
+    #TODO: base on generic ibgp graph, rather than bgp graph
+    g_bgp = anm['bgp']
+    g_phy = anm['phy']
+    g_ibgpv4 = anm.add_overlay("ibgp_v4", directed=True)
+    ipv4_nodes = set(g_phy.nodes("is_router", "use_ipv4"))
+    g_ibgpv4.add_nodes_from((n for n in g_bgp if n in ipv4_nodes),
+            retain = ["ibgp_level", "ibgp_l2_cluster", "ibgp_l3_cluster"] )
+    g_ibgpv4.add_edges_from(g_bgp.edges(type="ibgp"), retain="direction")
+
+def build_ibgp_v6(anm):
+    #TODO: remove the bgp layer and have just ibgp and ebgp
+    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
+    #TODO: base on generic ibgp graph, rather than bgp graph
+    g_bgp = anm['bgp']
+    g_phy = anm['phy']
+    g_ibgpv6 = anm.add_overlay("ibgp_v6", directed=True)
+    ipv6_nodes = set(g_phy.nodes("is_router", "use_ipv6"))
+    g_ibgpv6.add_nodes_from((n for n in g_bgp if n in ipv6_nodes),
+            retain = ["ibgp_level", "ibgp_l2_cluster", "ibgp_l3_cluster"] )
+    g_ibgpv6.add_edges_from(g_bgp.edges(type="ibgp"), retain="direction")
+
+def build_ebgp_v4(anm):
+    #TODO: remove the bgp layer and have just ibgp and ebgp
+    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
+    g_ebgp = anm['ebgp']
+    g_phy = anm['phy']
+    g_ebgpv4 = anm.add_overlay("ebgp_v4", directed=True)
+    ipv4_nodes = set(g_phy.nodes("is_router", "use_ipv4"))
+    g_ebgpv4.add_nodes_from(n for n in g_ebgp if n in ipv4_nodes)
+    g_ebgpv4.add_edges_from(g_ebgp.edges(), retain="direction")
+
+def build_ebgp_v6(anm):
+    #TODO: remove the bgp layer and have just ibgp and ebgp
+    # TODO: build from design rules, currently just builds from ibgp links in bgp layer
+    g_ebgp = anm['ebgp']
+    g_phy = anm['phy']
+    g_ebgpv6 = anm.add_overlay("ebgp_v6", directed=True)
+    ipv6_nodes = set(g_phy.nodes("is_router", "use_ipv6"))
+    g_ebgpv6.add_nodes_from(n for n in g_ebgp if n in ipv6_nodes)
+    g_ebgpv6.add_edges_from(g_ebgp.edges(), retain="direction")
+
+def build_ebgp(anm):
+    g_in = anm['input']
+    g_phy = anm['phy']
+    g_ebgp = anm.add_overlay("ebgp", directed=True)
+    g_ebgp.add_nodes_from(g_in.nodes("is_router"))
+    ebgp_edges = [e for e in g_in.edges() if not e.attr_equal("asn")]
+    g_ebgp.add_edges_from(ebgp_edges, bidirectional=True, type='ebgp')
+
+    ebgp_switches = [n for n in g_in.nodes("is_switch")
+            if not ank_utils.neigh_equal(g_phy, n, "asn")]
+    g_ebgp.add_nodes_from(ebgp_switches, retain=['asn'])
+    log.debug("eBGP switches are %s" % ebgp_switches)
+    g_ebgp.add_edges_from((e for e in g_in.edges()
+            if e.src in ebgp_switches or e.dst in ebgp_switches), bidirectional=True, type='ebgp')
+    ank_utils.aggregate_nodes(g_ebgp, ebgp_switches, retain="edge_id")
+    ebgp_switches = list(g_ebgp.nodes("is_switch")) # need to recalculate as may have aggregated
+    log.debug("aggregated eBGP switches are %s" % ebgp_switches)
+    exploded_edges = ank_utils.explode_nodes(g_ebgp, ebgp_switches,
+            retain="edge_id")
+    for edge in exploded_edges:
+        edge.multipoint = True
+
 def build_bgp(anm):
     """Build iBGP end eBGP overlays"""
     # eBGP
     g_in = anm['input']
     g_phy = anm['phy']
+
+    build_ebgp(anm)
+    build_ebgp_v4(anm)
+    build_ebgp_v6(anm)
+
+    """TODO: remove from here once compiler updated"""
     g_bgp = anm.add_overlay("bgp", directed=True)
     g_bgp.add_nodes_from(g_in.nodes("is_router"))
     ebgp_edges = [edge for edge in g_in.edges() if not edge.attr_equal("asn")]
@@ -338,6 +558,7 @@ def build_bgp(anm):
             retain="edge_id")
     for edge in exploded_edges:
         edge.multipoint = True
+    """TODO: remove up to here once compiler updated"""
 
 # now iBGP
     ank_utils.copy_attr_from(g_in, g_bgp, "ibgp_level")
@@ -425,6 +646,9 @@ def build_bgp(anm):
         for interface in node.interfaces():
             interface.multipoint = any(e.multipoint for e in interface.edges())
 
+    build_ibgp_v4(anm)
+    build_ibgp_v6(anm)
+
 def build_ipv6(anm):
     """Builds IPv6 graph, using nodes and edges from IPv4 graph"""
     import autonetkit.plugins.ipv6 as ipv6
@@ -434,7 +658,6 @@ def build_ipv6(anm):
     g_ipv6.add_nodes_from(
         g_ip, retain="collision_domain")  # retain if collision domain or not
     g_ipv6.add_edges_from(g_ip.edges())
-    autonetkit.update_http(anm)
 
     ipv6.allocate_ips(g_ipv6)
 
@@ -475,6 +698,8 @@ def manual_ipv4_infrastructure_allocation(anm):
                 subnet = netaddr.IPNetwork(cidr_string)
                 collision_domain.subnet = subnet
 
+    #TODO: assign directly to interfaces
+
     # also need to form aggregated IP blocks (used for e.g. routing prefix
     # advertisement)
     infra_blocks = {}
@@ -522,7 +747,7 @@ def build_ip(anm):
     for edge in edges_to_split:
         edge.split = True # mark as split for use in building nidb
     split_created_nodes = list(
-        ank_utils.split(g_ip, edges_to_split, retain=['edge_id', 'split']))
+        ank_utils.split(g_ip, edges_to_split, retain=['edge_id', 'split'], id_prepend = "cd"))
     for node in split_created_nodes:
         node['graphics'].x = ank_utils.neigh_average(g_ip, node, "x",
                                                      g_graphics) + 0.1 # temporary fix for gh-90
@@ -548,8 +773,9 @@ def build_ip(anm):
         graphics_node = g_graphics.node(node)
         graphics_node.device_type = "collision_domain"
         if not node.is_switch:
-            label = "_".join(
-                sorted(ank_utils.neigh_attr(g_ip, node, "label", g_phy)))
+            # use node sorting, as accomodates for numeric/string names
+            neighbors = sorted(neigh for neigh in node.neighbors())
+            label = "_".join(neigh.label for neigh in neighbors) 
             cd_label = "cd_%s" % label  # switches keep their names
             node.label = cd_label
             node.cd_id = cd_label
@@ -565,7 +791,6 @@ def build_ipv4(anm, infrastructure=True):
     # Copy ASN attribute chosen for collision domains (used in alloc algorithm)
     ank_utils.copy_attr_from(g_ip, g_ipv4, "asn", nbunch = g_ipv4.nodes("collision_domain"))
     g_ipv4.add_edges_from(g_ip.edges())
-    autonetkit.update_http(anm)
 
     #TODO: need to set allocate_ipv4 by default in the readers
     if g_in.data.alloc_ipv4_infrastructure is False:
@@ -582,7 +807,10 @@ def build_ipv4(anm, infrastructure=True):
         ipv4.allocate_ips(g_ipv4, infrastructure = False, loopbacks = True)
         #ank_utils.save(g_ipv4)
 
-    autonetkit.update_http(anm)
+    #TODO: need to also support secondary_loopbacks for IPv6
+    ipv4.allocate_ips(g_ipv4, infrastructure = False, loopbacks = False,
+            secondary_loopbacks = True)
+
 
     #TODO: replace this with direct allocation to interfaces in ip alloc plugin
     for node in g_ipv4.nodes("is_l3device"):
@@ -595,28 +823,30 @@ def build_ipv4(anm, infrastructure=True):
                 interface.subnet = edge.dst.subnet # from collision domain
 
     # TODO: also map loopbacks to loopback interface 0
-    autonetkit.update_http(anm)
 
 def build_phy(anm):
     """Build physical overlay"""
     g_in = anm['input']
     g_phy = anm['phy']
     g_phy.add_nodes_from(g_in, retain=['label', 'update', 'device_type', 'asn',
-                         'device_subtype', 'platform', 'host', 'syntax'])
+        'specified_int_names',
+        'device_subtype', 'platform', 'host', 'syntax'])
     if g_in.data.Creator == "Topology Zoo Toolset":
         ank_utils.copy_attr_from(g_in, g_phy, "Network")
 
+    if g_in.data.Creator == "Maestro":
+        g_phy.data.management_subnet = g_in.data.management_subnet 
+        g_phy.data.management_prefixlen = g_in.data.management_prefixlen 
+
     g_phy.add_edges_from(g_in.edges(type="physical"))
     # TODO: make this automatic if adding to the physical graph?
     g_phy.allocate_interfaces() 
 
-    specified_int_names = g_in.data.specified_int_names
-    if specified_int_names:
-        for node in g_phy:
-            for interface in node:
-                edge = interface.edges()[0]
-                directed_edge = anm['input_directed'].edge(edge)
-                interface.name = directed_edge.name
+    for node in g_phy.nodes("specified_int_names"):
+        for interface in node:
+            edge = interface.edges()[0]
+            directed_edge = anm['input_directed'].edge(edge)
+            interface.name = directed_edge.name
 
 def build_conn(anm):
     """Build connectivity overlay"""
@@ -644,13 +874,19 @@ def build_ospf(anm):
     """
     import netaddr
     g_in = anm['input']
+    # add regardless, so allows quick check of node in anm['ospf'] in compilers
     g_ospf = anm.add_overlay("ospf")
+
+    if not any(n.igp == "ospf" for n in g_in):
+        log.debug("No OSPF nodes")
+        return
+
     g_ospf.add_nodes_from(g_in.nodes("is_router", igp = "ospf"), retain=['asn'])
     g_ospf.add_nodes_from(g_in.nodes("is_switch"), retain=['asn'])
     g_ospf.add_edges_from(g_in.edges(), retain=['edge_id'])
 
     ank_utils.copy_attr_from(g_in, g_ospf, "ospf_area", dst_attr="area")
-    ank_utils.copy_edge_attr_from(g_in, g_ospf, "ospf_cost", dst_attr="cost")
+    ank_utils.copy_edge_attr_from(g_in, g_ospf, "ospf_cost", dst_attr="cost",  type=float)
 
     ank_utils.aggregate_nodes(g_ospf, g_ospf.nodes("is_switch"),
                               retain="edge_id")
@@ -773,11 +1009,13 @@ def ip_to_net_ent_title_ios(ip_addr):
 def build_isis(anm):
     """Build isis overlay"""
     g_in = anm['input']
+    # add regardless, so allows quick check of node in anm['isis'] in compilers
+    g_isis = anm.add_overlay("isis") 
+
     if not any(n.igp == "isis" for n in g_in):
         log.debug("No ISIS nodes")
         return
     g_ipv4 = anm['ipv4']
-    g_isis = anm.add_overlay("isis")
     g_isis.add_nodes_from(g_in.nodes("is_router", igp = "isis"), retain=['asn'])
     g_isis.add_nodes_from(g_in.nodes("is_switch"), retain=['asn'])
     g_isis.add_edges_from(g_in.edges(), retain=['edge_id'])
@@ -807,7 +1045,6 @@ def build_isis(anm):
             interface.metric = edge.metric
             interface.multipoint = edge.multipoint
 
-
 def update_messaging(anm):
     """Sends ANM to web server"""
     log.debug("Sending anm to messaging")
