@@ -10,7 +10,7 @@ from collections import defaultdict
 import netaddr
 import functools
 
-messaging = autonetkit.ank_messaging.AnkMessaging()
+messaging = autonetkit.ank_messaging
 
 #TODO: allow slack in allocations: both for ASN (group level), and for collision domains to allow new nodes to be easily added
 
@@ -435,8 +435,10 @@ def assign_asn_to_interasn_cds(g_ip):
 
     return
 
-def allocate_ips(g_ip, infrastructure = True, loopbacks = True):
+def allocate_ips(g_ip, infrastructure = True, loopbacks = True, secondary_loopbacks = False):
+    #TODO: tidy up the below comment and make all arguments default to False
     """Can disable infrastructure, eg for ipv6, still want to alloc ipv4 loopbacks for router ids"""
+    loopback_tree = []
     if loopbacks:
         log.info("Allocating v4 Primary Host loopback IPs")
         ip_tree = IpTree("192.168.1.0")
@@ -445,27 +447,27 @@ def allocate_ips(g_ip, infrastructure = True, loopbacks = True):
         loopback_tree = ip_tree.json()
     # json.dumps(ip_tree.json(), cls=autonetkit.ank_json.AnkEncoder, indent = 4)
         #body = json.dumps({"ip_allocations": jsontree})
-        #messaging.publish_compressed("www", "client", body)
         ip_tree.assign()
         g_ip.data.loopback_blocks = ip_tree.group_allocations()
 
-    log.debug("Allocating v4 Secondary Host loopback IPs")
-    ip_tree = IpTree("172.16.0.0")
-    secondary_loopbacks = []
-    for n in g_ip.nodes():
-        for i in n.interfaces("is_loopback"):
-            secondary_loopbacks.append(i)
+    secondary_loopback_tree = []
+    if secondary_loopbacks:
+        log.info("Allocating v4 Secondary Host loopback IPs")
+        #TODO: trim g_ip.nodes() to g_ip
+        ip_tree = IpTree("172.16.0.0")
+        secondary_loopbacks = [i for n in g_ip.nodes()
+                for i in n.loopback_interfaces
+                if not i.is_loopback_zero]
 
-    ip_tree.add_nodes(secondary_loopbacks)
-    ip_tree.build()
-    secondary_loopback_tree = ip_tree.json()
-   # json.dumps(ip_tree.json(), cls=autonetkit.ank_json.AnkEncoder, indent = 4)
-    #body = json.dumps({"ip_allocations": jsontree})
-    #messaging.publish_compressed("www", "client", body)
-    ip_tree.assign()
-    #g_ip.data.loopback_blocks = ip_tree.group_allocations()
+        ip_tree.add_nodes(secondary_loopbacks)
+        ip_tree.build()
+        secondary_loopback_tree = ip_tree.json()
+    # json.dumps(ip_tree.json(), cls=autonetkit.ank_json.AnkEncoder, indent = 4)
+        ip_tree.assign()
+        #g_ip.data.loopback_blocks = ip_tree.group_allocations()
 
 
+    cd_tree = []
     if infrastructure:
         log.info("Allocating v4 Infrastructure IPs")
         ip_tree = IpTree("10.0.0.0")
@@ -474,21 +476,33 @@ def allocate_ips(g_ip, infrastructure = True, loopbacks = True):
         ip_tree.build()
         cd_tree = ip_tree.json()
         ip_tree.assign()
+
+        total_tree = {
+                'name': "ip",
+                'children': 
+                [loopback_tree, secondary_loopback_tree, cd_tree],
+                #[cd_tree],
+                #[secondary_loopback_tree],
+                    #[loopback_tree],
+                }
+        jsontree = json.dumps(total_tree, cls=autonetkit.ank_json.AnkEncoder, indent = 4)
+
+#TODO: use seperate call in post to pass "ip_allocations" rather than in json
+
     else:
         cd_tree = {}
 
     total_tree = {
             'name': "ip",
             'children': 
-            #[loopback_tree, secondary_loopback_tree, cd_tree],
-            [cd_tree],
+            [loopback_tree, secondary_loopback_tree, cd_tree],
+            #[cd_tree],
             #[secondary_loopback_tree],
                 #[loopback_tree],
             }
     jsontree = json.dumps(total_tree, cls=autonetkit.ank_json.AnkEncoder, indent = 4)
+    messaging.publish_data(jsontree, "ip_allocations")
 
-    body = json.dumps({"ip_allocations": jsontree})
-    messaging.publish_compressed("www", "client", body)
 
 #TODO: need to update with loopbacks if wish to advertise also - or subdivide blocks?
     g_ip.data.infra_blocks = ip_tree.group_allocations()
