@@ -1,5 +1,39 @@
 //TODO: see if can use underscore.js for other operations, to simplify mapping, iterationl etc
 //List concat based on http://stackoverflow.com/questions/5080028
+//
+
+var g_groupings = chart.append("svg:g")
+.attr("id", "g_groupings");
+
+var g_links = chart.append("svg:g")
+.attr("id", "g_links");
+
+var g_nodes = chart.append("svg:g")
+.attr("id", "g_nodes");
+
+var g_node_labels = chart.append("svg:g")
+.attr("id", "g_node_labels");
+
+var g_link_labels = chart.append("svg:g")
+.attr("id", "g_link_labels");
+
+var g_traces = chart.append("svg:g")
+.attr("id", "g_highlights");
+
+var g_highlights = chart.append("svg:g")
+.attr("id", "g_highlights");
+
+
+var g_interfaces = chart.append("svg:g")
+.attr("id", "g_interfaces");
+
+//To store svg defs for icons
+chart.append("defs")
+
+var icon_width = 45;
+var icon_height = 45;
+
+
 
 
 var jsondata;
@@ -15,8 +49,6 @@ ws.onclose = function () {
     $("#websocket_icon").html(' <font color ="red"> <i class="icon-remove-sign " title="WebSocket Disconnected. Reload page to reconnect."></i></font>');
 };
 
-var icon_width = 45;
-var icon_height = 45;
 
 //TODO: make "phy" default selected
 
@@ -31,6 +63,11 @@ var node_label_id = "id";
 var edge_group_id = ""; //TODO: rename edge_label_id
 var interface_label_id = "";
 
+var interface_overlay_groupings = {
+    'ospf': 'area',
+    'vrf': 'vrf_name',
+}
+
 starting_hosts = [];
 
 ws.onmessage = function (evt) {
@@ -95,6 +132,7 @@ highlight_edges = [];
 
 var apply_highlight = function(data){
 
+
     highlight_nodes = _.map(data['nodes'], function(n) {
         return nodes_by_id[n];
     })
@@ -107,9 +145,25 @@ var apply_highlight = function(data){
         target_index = nodes.indexOf(target);
 
         return {'source': src_index, 'target': target_index};
-    }
-    )
+    })
+
+    if ((data.nodes.length > 0) || (data.edges.length > 0)) {
         redraw();
+    }
+
+
+    //draw paths after redraw -> z-ordering means overlayed
+    pathinfo = []; //reset
+    if (data.paths.length > 0) {
+        for (index in data.paths) {
+            path = data.paths[index];
+            //path_nodes = _.map(path, function(n) {
+                //return nodes_by_id[n];
+            //})
+            pathinfo.push(path);
+        }
+        redraw_paths();
+    }
 }
 
 var load_ip_allocations = function(d) {
@@ -140,7 +194,7 @@ function redraw_ip_allocations() {
         nodes = []; //otherwise have single root node always present
     }
 
-    var node = chart.selectAll("g.node")
+    var node = g_nodes.selectAll("g.node")
         .data(nodes, name)
         node.enter().append("svg:g")
         .attr("transform", function(d) { return "translate(" + (d.y + 80) + "," + d.x +  ")"; })
@@ -207,7 +261,7 @@ function redraw_ip_allocations() {
         .style("fill-opacity", 1e-6);
 
     // Update the links…
-    var link = chart.selectAll("path.link")
+    var link = g_links.selectAll("path.link")
         .data(layout.links(nodes, name), function(d) { return d.target.id; });
 
     // Enter any new links at the parent's previous position.
@@ -329,7 +383,7 @@ var clear_label = function() {
     status_label.html("");
 }
 
-var trace_paths = chart.append("svg:g")
+var trace_paths = g_highlights.append("svg:g")
 .attr("id", "path");
 
 var nodes = d3.map;
@@ -445,10 +499,9 @@ var interface_y = function(d) {
     return node_y(d.node) + offset_y - interface_height/2;
 }
 
-
 var groupPath = function(d) {
 
-    if (display_interfaces && overlay_id == "ospf") {
+    if (display_interfaces && overlay_id in interface_overlay_groupings) {
         if (d.values.length  == 1) {
             //This shouldn't occur for single ospf interface!
             node = d.values[0];
@@ -496,7 +549,7 @@ var groupPath = function(d) {
 
     if (d.values.length  == 1) {
         node = d.values[0];
-        offset = 2;
+        offset = 5;
         retval =  "M" ;
         retval += (node.x - offset  + icon_offset + x_offset) + "," + (node.y - offset + icon_offset + y_offset) + "L";
         retval += (node.x + offset  + icon_offset + x_offset) + "," + (node.y - offset + icon_offset + y_offset) + "L";
@@ -545,7 +598,7 @@ var groupPath = function(d) {
 
 var path_x = function(d) {
     node = nodes_by_id[d];
-    return node.x+ icon_width/2 + x_offset;
+    return node.x + icon_width/2 + x_offset;
 }
 
 var path_y = function(d) {
@@ -598,6 +651,7 @@ var link_info = function(d) {
     text = source.id + " - " + target.id; //TODO: make sure all have labels from graphics appended overlay
 
     for (attr in d) {
+        //TODO use list membership test here instead
         if (d[attr] != null && d[attr] != "None" && attr != "source" & attr != "target" && attr != "_interfaces" && attr != "edge_id") {
             text += ", " + attr + ": " + d[attr];
         }
@@ -624,23 +678,28 @@ var interface_info = function(d) {
 //Markers from http://bl.ocks.org/1153292
 // Used for arrow-heads
 // Per-type markers, as they don't inherit styles.
-chart.append("svg:defs").selectAll("marker")
-.data(["link_edge"])
+//TODO: create separate markers for traces and links
+//TODO: set marker properties by css
+//g_traces.append("svg:defs").selectAll("marker")
+//TODO: check this still works
+chart.select("defs").selectAll("marker")
+.data(["path_marker", "path_verified_marker"])
 .enter().append("svg:marker")
 .attr("id", String)
-.attr("viewBox", "0 0 12 12")
-.attr("refX", 35)
-.attr("refY", -4.5)
-.attr("overflow", "visible")
-.attr("markerWidth", 7)
-.attr("markerHeight", 7)
+.attr("refX", 2.4)
+.attr("refY", 2)
+//.attr("fill", "rgb(25,52,65)")
+//.attr("stroke", "rgb(25,52,65)")
+.attr("markerWidth", 20)
+.attr("markerHeight", 10)
+//.attr("markerUnits", "userSpaceOnUse")
 .attr("orient", "auto")
 .append("svg:path")
-.attr("d", "M0,-5L10,0L0,5");
+.attr("d", "M0,0 V4 L2,2 Z");
 
 var marker_end  = function(d) {
     if (jsondata.directed) {
-        return "url(#link_edge)";
+        return "url(#path_marker)";
     }
     return "";
 }
@@ -875,7 +934,8 @@ $(document).keydown(function(e){
 var group_attr = "asn";
 
 var group_info = function(d) {
-    if (overlay_id == "ospf") {
+    if (overlay_id in interface_overlay_groupings) {
+        //string tuple of "asn,grouping_attr"
         var data = d.key.split(",");
         text = ("Group: <ul><li>" + data[0] +  "</li><li>area: " + data[1] + "</li></ul>");
     } else {
@@ -894,13 +954,15 @@ var node_group_id = function(d) {
     else if (overlay_id == "conn") {
         group_attr = "device";
     }
-    else if (overlay_id == "ospf") {
-        return ([d['asn'], d['area']]);
-    }
     else if (overlay_id == "vrf") {
-        return ([d['asn'], d['vrf']]);
+        group_attr = "vrf";
     }
-    else if (overlay_id == "bgp") {
+    else if (overlay_id in interface_overlay_groupings) {
+        //TODO: don't want to have (asn, vrf) tuple for vrfs: just want (vrf)
+        attr = interface_overlay_groupings[overlay_id];
+        return ([d['asn'], d[attr]]);
+    }
+    else if (overlay_id == "bgp" || overlay_id == "ibgp_v4" || overlay_id == "ibgp_v6") {
         return (["ASN " + d['asn'], d['ibgp_l3_cluster'], d['ibgp_l2_cluster']]);
     }
 
@@ -1012,6 +1074,11 @@ var line_opacity = function(x) {
 var node_attributes = [];
 var edge_attributes = [];
 
+
+
+
+
+
 function redraw() {
     //TODO: tidy this up, not all functions need to be in here, move out those that do, and only pass required params. also avoid repeated calculations.
 
@@ -1138,30 +1205,30 @@ function redraw() {
 
 
     var interface_area = function(d) {
+        //TODO: should this only be called if display_interfaces && overlay_id in interface_overlay_groupings?
         interface_id = d.interface;
         asn = d.node['asn'];
-        area = d.node._interfaces[interface_id]['area'];
+        attr = interface_overlay_groupings[overlay_id];
+        area = d.node._interfaces[interface_id][attr];
         return asn + "," + area;
     }
 
     interface_attr_groups = d3.nest().key( interface_area ).entries(interface_data);
 
-    if (display_interfaces && overlay_id == "ospf") {
+    if (display_interfaces && overlay_id in interface_overlay_groupings) {
         node_attr_groups = interface_attr_groups;
     }
 
     var hull_stroke_width = function() {
-
-        if (display_interfaces && overlay_id == "ospf") {
+        if (display_interfaces && overlay_id in interface_overlay_groupings) {
             return 25;
         }
         return 80;
-
     }
 
 
     //TODO: make group path change/exit with node data
-    groupings = chart.selectAll(".attr_group")
+    groupings = g_groupings.selectAll(".attr_group")
         .data(node_attr_groups)
 
         groupings.enter().insert("path")
@@ -1201,7 +1268,7 @@ function redraw() {
 
     //TODO: filter the json data x and y ranges: store in nodes, and use this for the image plotting
 
-    node_highlight = chart.selectAll(".node_highlight")
+    node_highlight = g_highlights.selectAll(".node_highlight")
         .data(highlight_nodes, function(d) { return d.id;})
 
         node_highlight.enter().append("svg:rect")
@@ -1226,17 +1293,18 @@ function redraw() {
         .remove();
 
 
-    var line = chart.selectAll(".link_edge")
-        .data(jsondata.links, edge_id)
+    var line = g_links.selectAll(".link_edge")
+        .data(jsondata.links, 
+                function(d) { return d.source + "_" +  d.target})
 
         //line.enter().append("line")
         line.enter().append("svg:path")
         .attr("class", "link_edge")
         .style("opacity", line_opacity)
-        .attr("id", 
-                function(d) { 
-                    return "path"+d.source+"_"+d.target; 
-                }) 
+        //.attr("id", 
+                //function(d) { 
+                    //return "path"+d.source+"_"+d.target; 
+                //}) 
     .attr("d", graph_edge)
         .style("stroke-width", function() {
             //TODO: use this stroke-width function on mouseout too
@@ -1246,7 +1314,8 @@ function redraw() {
             return 2;
         })
     //.attr("marker-end", marker_end)
-    .style("stroke", "rgb(103,109,244)")
+    //.style("stroke", "rgb(103,109,244)")
+    .style("stroke", "rgb(2,106 ,155)")
         //.style("fill", "rgb(113,119,254)")
         .style("fill", "none")
 
@@ -1259,7 +1328,8 @@ function redraw() {
         })
     .on("mouseout", function(){
         d3.select(this).style("stroke-width", "2");
-        d3.select(this).style("stroke", "rgb(103,109,244)");
+        //d3.select(this).style("stroke", "rgb(103,109,244)");
+        d3.select(this).style("stroke", "rgb(2,106 ,155)");
         d3.select(this).style("fill", "none");
         //d3.select(this).attr("marker-end", marker_end);
         clear_label();
@@ -1334,7 +1404,7 @@ function redraw() {
         .style("opacity",0)
         .duration(4000);
 
-    interface_icons = chart.selectAll(".interface_icon")
+    interface_icons = g_interfaces.selectAll(".interface_icon")
         //.data(interface_data) //TODO: check if need to provide an index
         //TODO: check if should return tuple of interface, node for uniqueness (esp for switching overlays)
         .data(interface_data, function(d) { return (d.node, d.interface);})
@@ -1343,18 +1413,20 @@ function redraw() {
             interfaces = d3.selectAll(".interface_icon");
         }
 
+    //TODO: fix issue with interface id changing -> interfaces appear/reappear
+
     interface_icons.enter().append("svg:rect")
         .attr("class", "interface_icon")
         .attr("width", interface_width)
         .attr("height", interface_height)
         .attr("x", interface_x)
         .attr("y", interface_y)
-        .style("opacity", 0)
+        //.style("opacity", 0)
 
         interface_icons
         //TODO: look if can return multiple attributes, ie x and y, from the same function, ie calculation
         .attr("fill", "rgb(6,120,155)")
-        .style("opacity", 0)
+        //.style("opacity", 0)
 
         .on("mouseover", function(d){
             highlight_interfaces(d);
@@ -1383,15 +1455,15 @@ function redraw() {
     interface_icons.transition()
         .attr("x", interface_x)
         .attr("y", interface_y)
-        .style("opacity", interface_opacity)
+        //.style("opacity", interface_opacity)
         .duration(500);
 
     interface_icons.exit().transition()
         .duration(500)
-        .style("opacity",0)
+        //.style("opacity",0)
         .remove();
 
-    interface_labels = chart.selectAll(".interface_label")
+    interface_labels = g_interfaces.selectAll(".interface_label")
         .data(interface_data, function(d) { return d.interface;})
 
         interface_labels.enter().append("text")
@@ -1401,7 +1473,7 @@ function redraw() {
         .attr("text-anchor", "middle") 
         .attr("font-family", "helvetica") 
         //.attr("font-size", "small") 
-        .attr("font-size", 8) 
+        .attr("font-size", 10) 
 
         //TODO: use a general accessor for x/y of nodes
         interface_labels 
@@ -1420,7 +1492,7 @@ function redraw() {
         .remove();
 
     //Link labels
-    link_labels = chart.selectAll(".link_label")
+    link_labels = g_link_labels.selectAll(".link_label")
         .data(jsondata.links, edge_id)
 
         link_labels.enter().append("text")
@@ -1460,12 +1532,35 @@ function redraw() {
         return d.label + d.network;
     }
 
-    var image = chart.selectAll(".device_icon")
-        .attr("xlink:href", icon)
-        .data(nodes, node_id);
+    var node_icon = function(d) {
+        retval = d.device_type;
+        if (d.device_subtype != null && d.device_subtype != "None") {
+            retval += "_" + d.device_subtype;
+        }
+        return retval;
+    }
 
-    image.enter().append("image")
+    //Append any new device icons found
+    device_type_subtypes = _.map(nodes, node_icon);
+    device_type_subtypes = _.uniq(device_type_subtypes);
+    chart.select("defs")
+        .selectAll(".icondef")
+        .data(device_type_subtypes, function(d){  return d;})
+        .enter()
+        .append("image")
+        .attr("class", "icondef")
+        .attr("xlink:href", function (d){ return "icons/" + d + ".svg";})
+        .attr("id", function(d) {return "icon_" + d})
+        .attr("width", icon_width) 
+        .attr("height", icon_height);
+
+    var image = g_nodes.selectAll(".device_icon")
+        //.attr("xlink:href", icon)
+        .data(nodes, function(d) { return d.id});
+
+    image.enter().append("use")
         .attr("class", "device_icon")
+        .attr("xlink:href", function(d) {return "#icon_" + node_icon(d)})
         .attr("x", function(d) { return d.x + x_offset; })
         .attr("y", function(d) { return d.y + y_offset; })
         .style("opacity", icon_opacity)
@@ -1473,7 +1568,7 @@ function redraw() {
         .attr("height", icon_height)
         .on("mouseover", function(d){
             node_info(d);
-            d3.select(this).attr("xlink:href", icon); //TODO: check why need to do this
+            //d3.select(this).attr("xlink:href", icon); //TODO: check why need to do this
         })
     .on("mouseout", function(){
         clear_label();
@@ -1484,8 +1579,9 @@ function redraw() {
         image
         .attr("width", icon_width)
         .attr("height", icon_height)
+        .attr("xlink:href", function(d) {return "#icon_" + node_icon(d)})
         .transition()
-        .attr("xlink:href", icon)
+        .attr("xlink:href", function(d) {return "#icon_" + node_icon(d)})
         .style("opacity", icon_opacity)
         .attr("x", function(d) { return d.x + x_offset; })
         .attr("y", function(d) { return d.y + y_offset; })
@@ -1506,8 +1602,8 @@ function redraw() {
         }
     });
 
-    device_labels = chart.selectAll(".device_label")
-        .data(nodes, node_id)
+    device_labels = g_node_labels.selectAll(".device_label")
+        .data(nodes, function(d) { return d.id});
 
         device_labels.enter().append("text")
         .attr("x", function(d) { return d.x + x_offset; })
@@ -1539,52 +1635,131 @@ function redraw() {
 
 function redraw_paths() {
 
-    var traceroute_line = d3.svg.line()
+    //TODO: paths need to be updated when graph changes... or perhaps fade out as no longer relevant if topology changes?
+    //TODO: set paths using css and transition style rather than all the attributes hard coded
+
+    //animation based on http://bl.ocks.org/duopixel/4063326
+    var svg_line = d3.svg.line()
         .x(path_x)
         .y(path_y)
         .interpolate("cardinal")
         .tension(0.7)
         ;
 
-    //TODO: paths need to be updated when graph changes... or perhaps fade out as no longer relevant if topology changes?
-    //TODO: set paths using css and transition style rather than all the attributes hard coded
+    trace_path = g_traces.selectAll(".trace_path")
+        .data(pathinfo, function(path) {
+            elements = path['path'];
+            return _.first(elements) + "_" + _.last(elements);;
+        })
+    
+    var path_total_length = function(d) {
+        return d.node().getTotalLength()
+    }
 
-    path2 = chart.selectAll(".trace_path")
-        .data(pathinfo)
+    var path_marker_end = function(d) { 
+        if ("verified" in d && d['verified'] == true) {
+            return "url(#path_verified_marker)";
+        }
+        return "url(#path_marker)";
+    }
 
-        path2.enter().append("svg:path")
-        .attr("d", traceroute_line)
+    var path_color = function(d) { 
+        if ("verified" in d && d['verified'] == true) {
+            return "rgb(0,128,64)";
+        }
+        return "rgb(25,52,65)";
+    }
+
+    trace_path.enter().append("svg:path")
+        .attr("d", function(d) { return svg_line(d['path'])})
         .attr("class", "trace_path")
-        .style("stroke-width", 10)
-        .style("stroke", "yellow")
+        .style("stroke-width", 5)
+        .style("stroke", "rgb(207,120,33)")
         .style("fill", "none")
-        //TODO: can use following to map to marker type
-        //.attr("marker-end", function(d) { return "url(#" + d.type + ")"; });
-        //.attr("marker-end", "url(#trace)")
-        .on("mouseover", function(d){
-            d3.select(this).style("stroke", "orange");
-            d3.select(this).style("stroke-width", "6");
-            path_info(d);
+        .attr("stroke-dasharray", function(d) {
+            return path_total_length(d3.select(this)) + " " + path_total_length(d3.select(this))})
+        .attr("stroke-dashoffset", function(d) {
+            return path_total_length(d3.select(this))})
+
+        trace_path
+        .transition()
+        .style("stroke", path_color)
+        .attr("d", function(d) { return svg_line(d['path'])})
+        .ease("linear")
+        .attr("stroke-dashoffset", 0)
+        .duration(1000)
+        .transition()
+        .attr("stroke-dasharray", "0")
+        .attr("marker-end", path_marker_end)
+        .duration(1)
+
+        trace_path
+          .on("mouseover", function(d){
+            d3.select(this).style("stroke", "rgb(242,130,6)");
+            d3.select(this).style("stroke-width", "4");
+            //path_info(d);
+            console.log(d.verified);
         })
     .on("mouseout", function(){
         d3.select(this).style("stroke-width", "3");
         d3.select(this).style("stroke", "yellow");
-        clear_label();
+        //clear_label();
     })
-    .transition()
-        .duration(2000)
-        .style("stroke-width", 3)
-        //.style("stroke", "rgb(0,154,138)")
-        .style("stroke", "orange")
-        .style("opacity", 50)
-        ;
-
-    path2.exit().transition()
-        .duration(2000)
-        .style("opacity",0)
-        .remove();
 
 
-}
+        //TODO: chain arrow-head to appear after path is drawn (ie fire fade-in after previous event)
 
 
+        //TODO: can use following to map to marker type
+        //.attr("marker-end", function(d) { return "url(#" + d.type + ")"; });
+        //.attr("marker-end", "url(#trace)")
+        //.on("mouseover", function(d){
+            //d3.select(this).style("stroke", "rgb(242,130,6)");
+            //d3.select(this).style("stroke-width", "4");
+            //path_info(d);
+        //})
+    //.on("mouseout", function(){
+        //d3.select(this).style("stroke-width", "3");
+        //d3.select(this).style("stroke", "yellow");
+        //clear_label();
+    //})
+
+    //.style("stroke-width", 5)
+    //.style("stroke", "rgb(25,52,65)")
+    //.style("opacity", 1) ;
+
+    trace_path.exit().transition()
+    .duration(1000)
+    .style("opacity",0)
+    .remove();
+
+    //device_labels = g_nodes.selectAll(".device_label")
+        //.transition()
+        //.duration(50)
+        //.style("opacity",0.5);
+
+    //image = g_nodes.selectAll(".device_icon")
+    //.transition()
+        //.duration(50)
+        //.style("stroke","red")
+        //.style("opacity",0.5);
+
+    //g_links.selectAll(".link_edge")
+                //.transition()
+        //.duration(50)
+        //.style("opacity",0.5);
+
+    //g_links.selectAll(".link_label")
+                //.transition()
+        //.duration(50)
+        //.style("opacity",0.5);
+
+    //groupings = g_groupings.selectAll(".attr_group")
+        //.transition()
+        //.duration(50)
+        //.style("opacity",0.1);
+        //
+
+    //window.print();
+
+    }
